@@ -5,9 +5,9 @@ This module provides CLI tools to modify or interact with datasets.
 For information regarding the other functionalities, refer to the `README.md`.
 """
 
-from .api.dataset import Dataset
+from webknossos import Mag, Dataset
+
+from .compress import compress_mag
 from .cubing import cubing
 from .downsampling import downsample_mags
-from .compress import compress_mag
-from .mag import Mag
 from .metadata import write_webknossos_metadata
