@@ -1,22 +1,22 @@
-from typing import List, Dict
+from argparse import ArgumentParser, Namespace
+from pathlib import Path
+from typing import Dict, List
 
-from versioning import get_available_version
-from wkcuber import downsample_mags
-from .compress import compress_mag_inplace
-from .metadata import refresh_metadata
-from .utils import (
+from webknossos import Mag
+
+from ._internal.utils import (
     add_distribution_flags,
     add_isotropic_flag,
-    setup_logging,
     add_sampling_mode_flag,
+    setup_logging,
+    setup_warnings,
 )
-from .mag import Mag
-from .converter import (
-    create_parser as create_conversion_parser,
-    main as auto_detect_and_run_conversion,
-)
-from argparse import Namespace, ArgumentParser
-from pathlib import Path
+from ._internal.versioning import get_available_version
+from .compress import compress_mag_inplace
+from .converter import create_parser as create_conversion_parser
+from .converter import main as auto_detect_and_run_conversion
+from .downsampling import downsample_mags
+from .metadata import refresh_metadata
 
 
 def detect_present_mags(target_path: Path) -> Dict[Path, List[Mag]]:
@@ -71,9 +71,7 @@ def create_parser() -> ArgumentParser:
     return parser
 
 
-def main(args: Namespace) -> None:
-    setup_logging(args)
-
+def cube_with_args(args: Namespace) -> None:
     if args.isotropic is not None:
         raise DeprecationWarning(
             "The flag 'isotropic' is deprecated. Consider using '--sampling_mode isotropic' instead."
@@ -106,7 +104,12 @@ def main(args: Namespace) -> None:
     refresh_metadata(args.target_path)
 
 
-if __name__ == "__main__":
+def main() -> None:
+    setup_warnings()
     args = create_parser().parse_args()
     setup_logging(args)
-    main(args)
+    cube_with_args(args)
+
+
+if __name__ == "__main__":
+    main()
