@@ -4,8 +4,12 @@
 Please use the Dataset API at `webknossos.dataset`.
 """
 
+from webknossos.utils import warn_deprecated
+
 from .bounding_box import BoundingBox
 from .dataset import Dataset
 from .layer import Layer
 from .mag_view import MagView
 from .view import View
+
+warn_deprecated("wkcuber.api", "webknossos")
