@@ -1,24 +1,23 @@
 import logging
-import time
 from argparse import ArgumentParser, Namespace
 from pathlib import Path
-from sklearn.preprocessing import LabelEncoder
-from typing import Tuple, Optional, Union, cast
+from typing import Any, Optional, Tuple, Union, cast
 
 import nibabel as nib
 import numpy as np
+from sklearn.preprocessing import LabelEncoder
+from webknossos import BoundingBox, DataFormat, Dataset, LayerCategoryType, Vec3Int
+from webknossos.utils import time_start, time_stop
 
-from webknossos.dataset import LayerCategoryType
-from webknossos.dataset.defaults import DEFAULT_WKW_FILE_LEN
-from wkcuber.api.dataset import Dataset
-from wkcuber.api.bounding_box import BoundingBox
-from wkcuber.utils import (
-    DEFAULT_WKW_VOXELS_PER_BLOCK,
+from ._internal.utils import (
+    add_data_format_flags,
+    add_voxel_size_flag,
     add_verbose_flag,
-    add_scale_flag,
     pad_or_crop_to_size_and_topleft,
     parse_bounding_box,
+    parse_path,
     setup_logging,
+    setup_warnings,
 )
 
 
@@ -33,7 +32,9 @@ def create_parser() -> ArgumentParser:
     )
 
     parser.add_argument(
-        "target_path", help="Output directory for the generated WKW dataset.", type=Path
+        "target_path",
+        help="Output directory for the generated WKW dataset.",
+        type=parse_path,
     )
 
     parser.add_argument(
@@ -88,14 +89,17 @@ def create_parser() -> ArgumentParser:
         default=None,
     )
 
-    add_scale_flag(parser, required=False)
+    add_voxel_size_flag(parser, required=False)
     add_verbose_flag(parser)
+    add_data_format_flags(parser)
 
     return parser
 
 
 def to_target_datatype(
-    data: np.ndarray, target_dtype: str, is_segmentation_layer: bool
+    data: np.ndarray,
+    target_dtype: Union[type, str, np.dtype],
+    is_segmentation_layer: bool,
 ) -> np.ndarray:
     if is_segmentation_layer:
         original_shape = data.shape
@@ -106,6 +110,7 @@ def to_target_datatype(
             .astype(np.dtype(target_dtype))
         )
 
+    factor: Any
     if data.dtype == np.dtype("float32"):
         factor = data.max()
     elif data.dtype == np.dtype("float64"):
@@ -125,15 +130,17 @@ def convert_nifti(
     target_path: Path,
     layer_name: str,
     dtype: str,
-    scale: Tuple[float, ...],
+    voxel_size: Tuple[float, ...],
+    data_format: DataFormat,
+    chunk_size: Vec3Int,
+    chunks_per_shard: Vec3Int,
     is_segmentation_layer: bool = False,
-    file_len: int = DEFAULT_WKW_FILE_LEN,
     bbox_to_enforce: Optional[BoundingBox] = None,
     use_orientation_header: bool = False,
     flip_axes: Optional[Union[int, Tuple[int, ...]]] = None,
 ) -> None:
-    voxels_per_cube = file_len * DEFAULT_WKW_VOXELS_PER_BLOCK
-    ref_time = time.time()
+    shard_size = chunk_size * chunks_per_shard
+    time_start(f"Converting of {source_nifti_path}")
 
     source_nifti = nib.load(str(source_nifti_path.resolve()))
 
@@ -145,11 +152,6 @@ def convert_nifti(
 
     cube_data = np.array(source_nifti.get_fdata())
 
-    max_cell_id_args = (
-        {"largest_segment_id": int(np.max(cube_data) + 1)}
-        if is_segmentation_layer
-        else {}
-    )
     category_type: LayerCategoryType = (
         "segmentation" if is_segmentation_layer else "color"
     )
@@ -177,10 +179,10 @@ def convert_nifti(
     if flip_axes:
         cube_data = np.flip(cube_data, flip_axes)
 
-    if scale is None:
-        scale = tuple(map(float, source_nifti.header["pixdim"][:3]))
+    if voxel_size is None:
+        voxel_size = tuple(map(float, source_nifti.header["pixdim"][:3]))
 
-    logging.info(f"Using scale: {scale}")
+    logging.info(f"Using voxel_size: {voxel_size}")
     cube_data = to_target_datatype(cube_data, dtype, is_segmentation_layer)
 
     # everything needs to be padded to
@@ -192,10 +194,9 @@ def convert_nifti(
             cube_data, target_size, target_topleft
         )
 
-    # Writing wkw compressed requires files of shape (voxels_per_cube, voxels_per_cube, voxels_per_cube)
+    # Writing wkw compressed requires files of shape (shard_size, shard_size, shard_size)
     # Pad data accordingly
-    padding_offset = voxels_per_cube - np.array(cube_data.shape[1:4]) % voxels_per_cube
-    padding_offset = (0, 0, 0)
+    padding_offset = shard_size - np.array(cube_data.shape[1:4]) % shard_size
     cube_data = np.pad(
         cube_data,
         (
@@ -206,23 +207,33 @@ def convert_nifti(
         ),
     )
 
-    wk_ds = Dataset.get_or_create(
-        target_path, scale=cast(Tuple[float, float, float], scale or (1, 1, 1))
+    wk_ds = Dataset(
+        target_path,
+        voxel_size=cast(Tuple[float, float, float], voxel_size or (1, 1, 1)),
+        exist_ok=True,
     )
-    wk_layer = wk_ds.get_or_add_layer(
-        layer_name,
-        category_type,
-        dtype_per_layer=np.dtype(dtype),
-        **max_cell_id_args,
-    )
-    wk_mag = wk_layer.get_or_add_mag("1", file_len=file_len)
-    wk_mag.write(cube_data)
-
-    logging.debug(
-        "Converting of {} took {:.8f}s".format(
-            source_nifti_path, time.time() - ref_time
+    wk_layer = (
+        wk_ds.get_or_add_layer(
+            layer_name,
+            category_type,
+            dtype_per_layer=np.dtype(dtype),
+            data_format=data_format,
+            largest_segment_id=int(np.max(cube_data) + 1),
+        )
+        if is_segmentation_layer
+        else wk_ds.get_or_add_layer(
+            layer_name,
+            category_type,
+            data_format=data_format,
+            dtype_per_layer=np.dtype(dtype),
         )
     )
+    wk_mag = wk_layer.get_or_add_mag(
+        "1", chunk_size=chunk_size, chunks_per_shard=chunks_per_shard
+    )
+    wk_mag.write(cube_data)
+
+    time_stop(f"Converting of {source_nifti_path}")
 
 
 def convert_folder_nifti(
@@ -230,7 +241,10 @@ def convert_folder_nifti(
     target_path: Path,
     color_subpath: str,
     segmentation_subpath: str,
-    scale: Tuple[float, ...],
+    voxel_size: Tuple[float, ...],
+    data_format: DataFormat,
+    chunk_size: Vec3Int,
+    chunks_per_shard: Vec3Int,
     use_orientation_header: bool = False,
     bbox_to_enforce: Optional[BoundingBox] = None,
     flip_axes: Optional[Union[int, Tuple[int, ...]]] = None,
@@ -266,7 +280,10 @@ def convert_folder_nifti(
                 target_path,
                 "color",
                 "uint8",
-                scale,
+                voxel_size,
+                data_format,
+                chunk_size,
+                chunks_per_shard,
                 is_segmentation_layer=False,
                 bbox_to_enforce=bbox_to_enforce,
                 use_orientation_header=use_orientation_header,
@@ -278,7 +295,10 @@ def convert_folder_nifti(
                 target_path,
                 "segmentation",
                 "uint8",
-                scale,
+                voxel_size,
+                data_format,
+                chunk_size,
+                chunks_per_shard,
                 is_segmentation_layer=True,
                 bbox_to_enforce=bbox_to_enforce,
                 use_orientation_header=use_orientation_header,
@@ -290,7 +310,10 @@ def convert_folder_nifti(
                 target_path,
                 path.stem,
                 "uint8",
-                scale,
+                voxel_size,
+                data_format,
+                chunk_size,
+                chunks_per_shard,
                 is_segmentation_layer=False,
                 bbox_to_enforce=bbox_to_enforce,
                 use_orientation_header=use_orientation_header,
@@ -309,20 +332,19 @@ def main(args: Namespace) -> None:
                 0 <= index <= 3
             ), "flip_axes parameter must only contain indices between 0 and 3."
 
-    conversion_args = {
-        "scale": args.scale,
-        "bbox_to_enforce": args.enforce_bounding_box,
-        "use_orientation_header": args.use_orientation_header,
-        "flip_axes": flip_axes,
-    }
-
     if source_path.is_dir():
         convert_folder_nifti(
             source_path,
             args.target_path,
             args.color_file,
             args.segmentation_file,
-            **conversion_args,
+            voxel_size=args.voxel_size,
+            data_format=args.data_format,
+            chunk_size=args.chunk_size,
+            chunks_per_shard=args.chunks_per_shard,
+            bbox_to_enforce=args.enforce_bounding_box,
+            use_orientation_header=args.use_orientation_header,
+            flip_axes=flip_axes,
         )
     else:
         convert_nifti(
@@ -330,12 +352,19 @@ def main(args: Namespace) -> None:
             args.target_path,
             args.layer_name,
             args.dtype,
+            voxel_size=args.voxel_size,
+            data_format=args.data_format,
+            chunk_size=args.chunk_size,
+            chunks_per_shard=args.chunks_per_shard,
             is_segmentation_layer=args.is_segmentation_layer,
-            **conversion_args,
+            bbox_to_enforce=args.enforce_bounding_box,
+            use_orientation_header=args.use_orientation_header,
+            flip_axes=flip_axes,
         )
 
 
 if __name__ == "__main__":
+    setup_warnings()
     args = create_parser().parse_args()
     setup_logging(args)
 
