@@ -1,31 +1,34 @@
+import logging
 from argparse import ArgumentParser, Namespace
 from os import path, sep
 from pathlib import Path
-from typing import Iterable, List, Any, Tuple, Dict, Set, Callable, cast, Optional
+from typing import Any, Callable, Dict, Iterable, List, Optional, Set, Tuple, cast
 
 from webknossos.dataset.properties import LayerViewConfiguration
-from .convert_knossos import (
-    main as convert_knossos,
-    create_parser as create_knossos_parser,
-)
-from .convert_nifti import main as convert_nifti, create_parser as create_nifti_parser
-from .cubing import (
-    cubing as cube_image_stack,
-    create_parser as create_image_stack_parser,
-    get_channel_and_sample_count_and_dtype,
-)
-from .image_readers import image_reader
-from .metadata import write_webknossos_metadata
-from .utils import (
-    find_files,
-    add_scale_flag,
-    logger,
+
+from ._internal.image_readers import image_reader
+from ._internal.utils import (
+    add_data_format_flags,
+    add_voxel_size_flag,
     add_verbose_flag,
-    setup_logging,
+    find_files,
+    get_channel_and_sample_iters_for_wk_compatibility,
     get_executor_args,
     is_wk_compatible_layer_format,
-    get_channel_and_sample_iters_for_wk_compatibility,
+    parse_path,
+    setup_logging,
+    setup_warnings,
 )
+from .convert_knossos import create_parser as create_knossos_parser
+from .convert_knossos import main as convert_knossos
+from .convert_nifti import create_parser as create_nifti_parser
+from .convert_nifti import main as convert_nifti
+from .cubing import create_parser as create_image_stack_parser
+from .cubing import cubing as cube_image_stack
+from .cubing import get_channel_and_sample_count_and_dtype
+from .metadata import write_webknossos_metadata
+
+logger = logging.getLogger(__name__)
 
 
 def create_parser() -> ArgumentParser:
@@ -40,11 +43,14 @@ def create_parser() -> ArgumentParser:
     )
 
     parser.add_argument(
-        "target_path", help="Output directory for the generated dataset.", type=Path
+        "target_path",
+        help="Output directory for the generated dataset.",
+        type=parse_path,
     )
 
-    add_scale_flag(parser)
+    add_voxel_size_flag(parser)
     add_verbose_flag(parser)
+    add_data_format_flags(parser)
 
     return parser
 
@@ -74,9 +80,7 @@ def get_source_files(
     elif not allows_single_file_input:
         return []
 
-    source_files = list(find_files(input_path, extensions))
-
-    return source_files
+    return [str(f) for f in find_files(input_path, extensions)]
 
 
 def all_files_of_same_type(input_files: List[str]) -> bool:
@@ -353,7 +357,7 @@ class ImageStackConverter(Converter):
             args, image_stack_parser, "target_mag"
         )
         put_default_from_argparser_if_not_present(
-            args, image_stack_parser, "wkw_file_len"
+            args, image_stack_parser, "chunks_per_shard"
         )
         put_default_from_argparser_if_not_present(
             args, image_stack_parser, "interpolation_mode"
@@ -421,12 +425,14 @@ class ImageStackConverter(Converter):
                         sample_index,
                         arg_dict.get("dtype"),
                         args.target_mag,
-                        args.wkw_file_len,
+                        args.data_format,
+                        args.chunk_size,
+                        args.chunks_per_shard,
                         args.interpolation_mode,
                         args.start_z,
                         args.skip_first_z_slices,
                         args.pad,
-                        args.scale,
+                        args.voxel_size,
                         executor_args,
                     )
 
@@ -541,16 +547,17 @@ def main(args: Namespace) -> None:
         exit(1)
     elif len(matching_converters) > 1:
         logger.info(
-            "Multiple converters found. Check if your source path contains multiple datasets."
+            f"Multiple converters found. Check if your source path contains multiple datasets. Converters: {matching_converters}"
         )
         exit(1)
 
     should_write_metadata = matching_converters[0].convert_input(args)
     if should_write_metadata:
-        write_webknossos_metadata(args.target_path, args.name, args.scale)
+        write_webknossos_metadata(args.target_path, args.name, args.voxel_size)
 
 
 if __name__ == "__main__":
+    setup_warnings()
     parsed_args = create_parser().parse_args()
     setup_logging(parsed_args)
 
