@@ -1,29 +1,28 @@
-from pathlib import Path
-
-from typing import Optional
-
 from argparse import ArgumentParser, Namespace
-import os
+from pathlib import Path
+from typing import Optional, Union
 
-from wkcuber.api.dataset import Dataset
-from wkcuber.downsampling_utils import SamplingModes
-from .mag import Mag
+from webknossos import Dataset, Mag, Vec3Int, SamplingModes
 
-from .utils import (
-    add_verbose_flag,
+from ._internal.utils import (
     add_distribution_flags,
     add_interpolation_flag,
     add_isotropic_flag,
-    setup_logging,
     add_sampling_mode_flag,
+    add_verbose_flag,
     get_executor_args,
+    parse_path,
+    setup_logging,
+    setup_warnings,
 )
 
 
 def create_parser() -> ArgumentParser:
     parser = ArgumentParser()
 
-    parser.add_argument("path", help="Directory containing the dataset.", type=Path)
+    parser.add_argument(
+        "path", help="Directory containing the dataset.", type=parse_path
+    )
 
     parser.add_argument(
         "--layer_name",
@@ -53,7 +52,7 @@ def create_parser() -> ArgumentParser:
         "equal or larger than 8. "
         "The default value is calculated depending on the dataset size. In the lowest Mag, the size will be "
         "smaller than 100vx per dimension",
-        type=int,
+        type=str,
         default=None,
     )
 
@@ -62,7 +61,7 @@ def create_parser() -> ArgumentParser:
         help="Specify an explicit anisotropic target magnification (e.g., --anisotropic_target_mag 2-2-1)."
         "All magnifications until this target magnification will be created. Consider using --anisotropic "
         "instead which automatically creates multiple anisotropic magnifications depending "
-        "on the dataset's scale",
+        "on the dataset's voxel_size",
         type=str,
     )
 
@@ -103,12 +102,19 @@ def downsample_mags(
     from_mag: Optional[Mag] = None,
     max_mag: Optional[Mag] = None,
     interpolation_mode: str = "default",
-    buffer_edge_len: Optional[int] = None,
+    buffer_shape: Optional[Vec3Int] = None,
     compress: bool = True,
     args: Optional[Namespace] = None,
-    sampling_mode: str = SamplingModes.ANISOTROPIC,
+    sampling_mode: Union[str, SamplingModes] = SamplingModes.ANISOTROPIC,
     force_sampling_scheme: bool = False,
 ) -> None:
+    """
+    Argument `path` expects the directory containing the dataset.
+    Argument `layer_name` expects the name of the layer (color or segmentation).
+    Argument `from_mag` expects the resolution to base downsampling on.
+
+    For the other parameters see the CLI help or `Layer.downsample` and `Layer.downsampling_mag`.
+    """
     assert layer_name and from_mag or not layer_name and not from_mag, (
         "You provided only one of the following "
         "parameters: layer_name, from_mag but both "
@@ -118,23 +124,27 @@ def downsample_mags(
         " (e.g dataset/color/1)."
     )
     if not layer_name or not from_mag:
-        layer_name = os.path.basename(os.path.dirname(path))
-        from_mag = Mag(os.path.basename(path))
+        layer_name = path.parent.name
+        from_mag = Mag(path.name)
         path = path.parent.parent
 
-    Dataset(path).get_layer(layer_name).downsample(
+    assert layer_name is not None  # for mypy
+    assert from_mag is not None  # for mypy
+
+    Dataset.open(path).get_layer(layer_name).downsample(
         from_mag=from_mag,
         max_mag=max_mag,
         interpolation_mode=interpolation_mode,
         compress=compress,
         sampling_mode=sampling_mode,
-        buffer_edge_len=buffer_edge_len,
+        buffer_shape=buffer_shape,
         force_sampling_scheme=force_sampling_scheme,
         args=args,
     )
 
 
 if __name__ == "__main__":
+    setup_warnings()
     args = create_parser().parse_args()
     setup_logging(args)
 
@@ -150,6 +160,11 @@ if __name__ == "__main__":
 
     from_mag = Mag(args.from_mag)
     max_mag = None if args.max is None else Mag(args.max)
+    buffer_shape = (
+        Vec3Int.full(args.buffer_cube_size)
+        if args.buffer_cube_size is not None
+        else None
+    )
 
     downsample_mags(
         path=args.path,
@@ -159,7 +174,7 @@ if __name__ == "__main__":
         interpolation_mode=args.interpolation_mode,
         compress=not args.no_compress,
         sampling_mode=args.sampling_mode,
-        buffer_edge_len=args.buffer_cube_size,
+        buffer_shape=buffer_shape,
         force_sampling_scheme=args.force_sampling_scheme,
         args=get_executor_args(args),
     )
